==> yaru-0.3.2.tar.gz: test/widget_test.dart <==
import 'dart:async';

import 'package:dbus/dbus.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:gsettings/gsettings.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:platform/platform.dart';
import 'package:yaru/yaru.dart';

import 'widget_test.mocks.dart';

@GenerateMocks([GSettings])
void main() {
  testWidgets('variant', (tester) async {
    await tester.pumpTheme(variant: YaruVariant.blue);
    final context = tester.element(find.byType(Container));
    expect(YaruTheme.of(context).variant, YaruVariant.blue);
  });

  group('gtk-theme', () {
    testWidgets('unknown', (tester) async {
      final settings = createMockGSettings(theme: '');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, null);
    });

    testWidgets('yaru', (tester) async {
      final settings = createMockGSettings(theme: 'Yaru');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.orange);
    });

    testWidgets('dark', (tester) async {
      final settings = createMockGSettings(theme: 'Yaru-dark');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.orange);
    });

    testWidgets('color', (tester) async {
      final settings = createMockGSettings(theme: 'Yaru-blue');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.blue);
    });

    testWidgets('change', (tester) async {
      final settings = createMockGSettings(theme: 'Yaru-blue');
      final keysChanged = StreamController<List<String>>(sync: true);
      addTearDown(keysChanged.close);
      when(settings.keysChanged).thenAnswer((_) => keysChanged.stream);

      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.blue);

      when(settings.get('gtk-theme'))
          .thenAnswer((_) async => const DBusString('Yaru-red'));
      keysChanged.add(['gtk-theme']);
      await tester.pump();
      expect(YaruTheme.of(context).variant, YaruVariant.red);
    });
  });

  group('desktop', () {
    testWidgets('unknown', (tester) async {
      final settings = createMockGSettings();
      await tester.pumpTheme(desktop: 'unknown', settings: settings);
      await untilCalled(settings.get('gtk-theme'));
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, isNull);
    });

    testWidgets('budgie', (tester) async {
      final settings = createMockGSettings();
      await tester.pumpTheme(desktop: 'budgie-desktop', settings: settings);
      await untilCalled(settings.get('gtk-theme'));
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.ubuntuBudgieBlue);
    });

    testWidgets('ubuntu', (tester) async {
      final settings = createMockGSettings();
      await tester.pumpTheme(desktop: 'GNOME:ubuntu', settings: settings);
      await untilCalled(settings.get('gtk-theme'));
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.orange);
    });

    testWidgets('kde', (tester) async {
      final settings = createMockGSettings();
      await tester.pumpTheme(desktop: 'KDE', settings: settings);
      await untilCalled(settings.get('gtk-theme'));
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.kubuntuBlue);
    });

    testWidgets('lubuntu', (tester) async {
      final settings = createMockGSettings();
      await tester.pumpTheme(desktop: 'LXQt', settings: settings);
      await untilCalled(settings.get('gtk-theme'));
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.lubuntuBlue);
    });

    testWidgets('mate', (tester) async {
      final settings = createMockGSettings();
      await tester.pumpTheme(desktop: 'MATE', settings: settings);
      await untilCalled(settings.get('gtk-theme'));
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.ubuntuMateGreen);
    });

    testWidgets('xubuntu', (tester) async {
      final settings = createMockGSettings();
      await tester.pumpTheme(desktop: 'XFCE', settings: settings);
      await untilCalled(settings.get('gtk-theme'));
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.xubuntuBlue);
    });
  });

  group('high contrast', () {
    testWidgets('on', (tester) async {
      await tester.pumpTheme(highContrast: true);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).highContrast, isTrue);
    });

    testWidgets('off', (tester) async {
      await tester.pumpTheme(highContrast: false);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).highContrast, isFalse);
    });
  });

  group('theme mode', () {
    testWidgets('light', (tester) async {
      await tester.pumpTheme(themeMode: ThemeMode.light);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).themeMode, ThemeMode.light);
    });

    testWidgets('dark', (tester) async {
      await tester.pumpTheme(themeMode: ThemeMode.dark);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).themeMode, ThemeMode.dark);
    });
  });
}

MockGSettings createMockGSettings({String theme = ''}) {
  final settings = MockGSettings();
  when(settings.keysChanged).thenAnswer((_) => const Stream.empty());
  when(settings.get('gtk-theme')).thenAnswer((_) async => DBusString(theme));
  return settings;
}

extension ThemeTester on WidgetTester {
  Future<void> pumpTheme({
    YaruVariant? variant,
    bool? highContrast,
    ThemeMode? themeMode,
    String desktop = '',
    GSettings? settings,
  }) async {
    final data = YaruThemeData(
      variant: variant,
      highContrast: highContrast,
      themeMode: themeMode,
    );
    await pumpWidget(MaterialApp(
      key: ValueKey(data),
      home: YaruTheme(
        child: Container(),
        data: data,
        platform: FakePlatform(
          operatingSystem: Platform.linux,
          environment: {'XDG_CURRENT_DESKTOP': desktop},
        ),
        settings: settings,
      ),
    ));
    return pump();
  }
}

==> yaru-0.4.3.tar.gz: test/widget_test.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:platform/platform.dart';
import 'package:yaru/yaru.dart';
import 'package:yaru/src/settings.dart';

import 'widget_test.mocks.dart';

@GenerateMocks([YaruSettings])
void main() {
  testWidgets('variant', (tester) async {
    await tester.pumpTheme(variant: YaruVariant.blue);
    final context = tester.element(find.byType(Container));
    expect(YaruTheme.of(context).variant, YaruVariant.blue);
  });

  group('gtk-theme', () {
    testWidgets('unknown', (tester) async {
      final settings = createMockSettings(theme: '');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, null);
    });

    testWidgets('yaru', (tester) async {
      final settings = createMockSettings(theme: 'Yaru');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.orange);
    });

    testWidgets('dark', (tester) async {
      final settings = createMockSettings(theme: 'Yaru-dark');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.orange);
    });

    testWidgets('color', (tester) async {
      final settings = createMockSettings(theme: 'Yaru-blue');
      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.blue);
    });

    testWidgets('change', (tester) async {
      final settings = createMockSettings(theme: 'Yaru-blue');
      final themeNameChanged = StreamController<String>(sync: true);
      addTearDown(themeNameChanged.close);
      when(settings.themeNameChanged)
          .thenAnswer((_) => themeNameChanged.stream);

      await tester.pumpTheme(settings: settings);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.blue);

      themeNameChanged.add('Yaru-red');
      await tester.pump();
      expect(YaruTheme.of(context).variant, YaruVariant.red);
    });
  });

  group('desktop', () {
    testWidgets('unknown', (tester) async {
      final settings = createMockSettings();
      await tester.pumpTheme(desktop: 'unknown', settings: settings);
      await untilCalled(settings.getThemeName());
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, isNull);
    });

    testWidgets('budgie', (tester) async {
      final settings = createMockSettings();
      await tester.pumpTheme(desktop: 'budgie-desktop', settings: settings);
      await untilCalled(settings.getThemeName());
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.ubuntuBudgieBlue);
    });

    testWidgets('ubuntu', (tester) async {
      final settings = createMockSettings();
      await tester.pumpTheme(desktop: 'GNOME:ubuntu', settings: settings);
      await untilCalled(settings.getThemeName());
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.orange);
    });

    testWidgets('kde', (tester) async {
      final settings = createMockSettings();
      await tester.pumpTheme(desktop: 'KDE', settings: settings);
      await untilCalled(settings.getThemeName());
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.kubuntuBlue);
    });

    testWidgets('lubuntu', (tester) async {
      final settings = createMockSettings();
      await tester.pumpTheme(desktop: 'LXQt', settings: settings);
      await untilCalled(settings.getThemeName());
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.lubuntuBlue);
    });

    testWidgets('mate', (tester) async {
      final settings = createMockSettings();
      await tester.pumpTheme(desktop: 'MATE', settings: settings);
      await untilCalled(settings.getThemeName());
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.ubuntuMateGreen);
    });

    testWidgets('xubuntu', (tester) async {
      final settings = createMockSettings();
      await tester.pumpTheme(desktop: 'XFCE', settings: settings);
      await untilCalled(settings.getThemeName());
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).variant, YaruVariant.xubuntuBlue);
    });
  });

  group('high contrast', () {
    testWidgets('on', (tester) async {
      await tester.pumpTheme(highContrast: true);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).highContrast, isTrue);
    });

    testWidgets('off', (tester) async {
      await tester.pumpTheme(highContrast: false);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).highContrast, isFalse);
    });
  });

  group('theme mode', () {
    testWidgets('light', (tester) async {
      await tester.pumpTheme(themeMode: ThemeMode.light);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).themeMode, ThemeMode.light);
    });

    testWidgets('dark', (tester) async {
      await tester.pumpTheme(themeMode: ThemeMode.dark);
      final context = tester.element(find.byType(Container));
      expect(YaruTheme.of(context).themeMode, ThemeMode.dark);
    });
  });

  testWidgets('theme data overrides', (tester) async {
    const extensions = <ThemeExtension>[];
    const pageTransitionsTheme = PageTransitionsTheme();
    const visualDensity = VisualDensity(horizontal: -4, vertical: -4);
    await tester.pumpTheme(
      extensions: extensions,
      pageTransitionsTheme: pageTransitionsTheme,
      useMaterial3: false,
      visualDensity: visualDensity,
    );
    final context = tester.element(find.byType(Container));
    final theme = YaruTheme.of(context);
    expect(theme.extensions, same(extensions));
    expect(theme.pageTransitionsTheme, same(pageTransitionsTheme));
    expect(theme.useMaterial3, isFalse);
    expect(theme.visualDensity, visualDensity);
  });
}

MockYaruSettings createMockSettings({String theme = ''}) {
  final settings = MockYaruSettings();
  when(settings.themeNameChanged).thenAnswer((_) => const Stream.empty());
  when(settings.getThemeName()).thenAnswer((_) async => theme);
  return settings;
}

extension ThemeTester on WidgetTester {
  Future<void> pumpTheme({
    YaruVariant? variant,
    bool? highContrast,
    ThemeMode? themeMode,
    Iterable<ThemeExtension<dynamic>>? extensions,
    PageTransitionsTheme? pageTransitionsTheme,
    bool? useMaterial3,
    VisualDensity? visualDensity,
    String desktop = '',
    YaruSettings? settings,
  }) async {
    final data = YaruThemeData(
      variant: variant,
      highContrast: highContrast,
      themeMode: themeMode,
      extensions: extensions,
      pageTransitionsTheme: pageTransitionsTheme,
      useMaterial3: useMaterial3,
      visualDensity: visualDensity,
    );
    await pumpWidget(
      MaterialApp(
        key: ValueKey(data),
        home: YaruTheme(
          child: Container(),
          data: data,
          platform: FakePlatform(
            operatingSystem: Platform.linux,
            environment: {'XDG_CURRENT_DESKTOP': desktop},
          ),
          settings: settings,
        ),
      ),
    );
    return pump();
  }
}
